import { Inject, Injectable, InjectionToken, Injector, NgModule } from '@angular/core';

var EAGER_PROVIDER = new InjectionToken('EAGER_PROVIDER');

function eagerProviderRegistration(token) {
    return {
        provide: EAGER_PROVIDER,
        useValue: token,
        multi: true
    };
}
function eagerProvider(provider) {
    var token = provider.provide || provider;
    return [provider, eagerProviderRegistration(token)];
}
function eagerProviders() {
    var providers = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        providers[_i] = arguments[_i];
    }
    return providers.map(eagerProvider).reduce(function (a, b) { return a.concat(b); }, []);
}

var EagerProviderLoaderService = (function () {
    function EagerProviderLoaderService() {
        this.loadedProviders = new Set();
    }
    EagerProviderLoaderService.prototype.loadProviders = function (injector) {
        var _this = this;
        // Obtain the set of tokens for the providers which need to be loaded eagerly.
        var tokens = injector.get(EAGER_PROVIDER, []);
        // Filter out all tokens for the providers which have already been loaded.
        // Afterwards use the injector to get each of the remaining provider tokens.
        // This forces the corresponding providers to be instantiated.
        tokens
            .filter(function (token) { return !_this.loadedProviders.has(token); })
            .forEach(function (token) {
            injector.get(token);
            _this.loadedProviders.add(token);
        });
    };
    return EagerProviderLoaderService;
}());
EagerProviderLoaderService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
EagerProviderLoaderService.ctorParameters = function () { return []; };

function getEagerProviderLoaderService(injector) {
    // Construct the injector stack from the current injector to the root injector.
    var injectorStack = [injector];
    while (injectorStack[injectorStack.length - 1].parent) {
        injectorStack.push(injectorStack[injectorStack.length - 1].parent);
    }
    // Starting at the root injector, descent in the injector tree to find the first injector that provides the EagerProviderLoaderService.
    var loaderService = null;
    while (!loaderService && injectorStack.length) {
        loaderService = injectorStack.pop().get(EagerProviderLoaderService, null);
    }
    return loaderService;
}
var EagerProviderLoaderModule = (function () {
    function EagerProviderLoaderModule(injector, eagerProviderTokens) {
        if (!eagerProviderTokens) {
            eagerProviderTokens = [];
        }
        // Retrieve the eager provider loader service that is closest to the root injector.
        var eagerProviderLoaderService = getEagerProviderLoaderService(injector);
        // Load the providers marked for eager loading for the current injector.
        eagerProviderLoaderService.loadProviders(injector);
    }
    return EagerProviderLoaderModule;
}());
EagerProviderLoaderModule.decorators = [
    { type: NgModule, args: [{
                providers: [EagerProviderLoaderService]
            },] },
];
/** @nocollapse */
EagerProviderLoaderModule.ctorParameters = function () { return [
    { type: Injector, },
    { type: Array, decorators: [{ type: Inject, args: [EAGER_PROVIDER,] },] },
]; };

export { EAGER_PROVIDER, eagerProviderRegistration, eagerProvider, eagerProviders, EagerProviderLoaderService, EagerProviderLoaderModule };
